// $Id: Cache.java,v 1.1 2001/08/15 18:51:50 drager Exp $

package org.daml.validator;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.io.*;
import java.net.*;

/**
 * This Class manages the maintainence of the Validator Cache.
 */

public class Cache {

    boolean start = true;
    int action = 0;
    HashSet uri = new HashSet();
    boolean all = false;
    String preferenceFile = "preferences.xml";
    String cachedir = "cache/";
    org.daml.validator.Preferences preferences = null;

    public static void usage()
    {
	System.out.println("Usage:");
	System.out.println("    [-cache <pathname>]");
	System.out.println("    [-pref <preference file>]");
	System.out.println("    {-add | -del} {<URL> | -f <filename> | -all}");
	System.out.println("");
	System.out.println("The default cache pathname is cache/");
	System.out.println("The default preference file is preferences.xml");
	System.out.println("");
	System.out.println("Example usage:");
	System.out.println("");
	System.out.println("To update all cache files use");
	System.out.println("  java org.daml.validator.Cache -add -all");
	System.out.println("");
	System.out.println("To add a list of URLs saved in a file called urls.txt (one per line) use");
	System.out.println("  java org.daml.validator.Cache -add -f urls.txt");
	System.out.println("");
	System.out.println("To delete http://www.daml.org/index.html from the cache use");
	System.out.println("  java org.daml.validator.Cache -del http://www.daml.org/index.html");
	
    }

    public Cache(String [] args)
    {
	parseArguments(args);
    }

    public void parseArguments(String [] args) {
	start = true;

	try {
	    int i = 0;

	    if (args[i].equals("-pref")) {
		preferenceFile = args[i+1];
		i += 2;
	    }

	    if (args[i].equals("-cache")) {
		cachedir = args[i+1];
		i += 2;
	    }

	    if (args[i].equals("-add")) {
		action = 0;
	    } else if (args[i].equals("-del")) {
		action = 1;
	    } else {
		start = false;
		return;
	    }

	    i += 1;

	    // Load Preferences
	    try {
		preferences = org.daml.validator.Preferences.parse(preferenceFile);
	    } catch (Exception e) {
		System.out.println("Exception occurred while parsing preference file.");
		System.out.println(e.toString());
		start = false;
		return;
	    }

	    if (args[i].equals("-f")) {
		start=parseFile(args[i+1], uri);
	    } else if (args[i].equals("-all")) {
		// All the files currently in the file cache.
		uri.addAll(preferences.cacheList);
	    } else {
		uri.add(args[i]);
	    }

	} catch (ArrayIndexOutOfBoundsException ex) {
	    start = false;
	}
    }

    private boolean parseFile(String filename, Set set)
    {
	try {
	    BufferedReader in = new BufferedReader(new FileReader(filename));
	    String line = in.readLine();
	    while (line != null) {
		set.add(line);
		line = in.readLine();
	    }
	} catch (Exception ex) {
	    System.out.println("An Exception occurred while reading "+filename);
	    System.out.println(ex);
	    return(false);
	}
	return(true);
    }

    /**
     * Main method for cache maintence functions.
     */
    public static void main (String args[])
    throws Exception
    {
	Cache cache = new Cache(args);
	if (cache.start) {
	    cache.run();
	} else {
	    cache.usage();
	}
    }

    private void run() 
	throws java.net.MalformedURLException
    {
	if (action == 1) {
	    System.out.println("Deleting Files:");
	    // Delete files...
	    for (Iterator i = uri.iterator(); i.hasNext(); ) {
		String u = (String)i.next();
		deleteCacheFile(u);
	    }
	    System.out.println("Done.");
	} else if (action == 0) {
	    // Add/update files
	    for (Iterator i = uri.iterator(); i.hasNext(); ) {
		String u = (String)i.next();
		createCacheFile(u);
	    }
	}
	// Save new preference file
	try {
	preferences.save(preferenceFile);
	} catch (java.io.IOException ex) {
	    System.out.println("Could not write preferences to file "+preferenceFile+".");
	    System.out.println(ex);
	}
    }

    // Have multiple add modes
    // Add only new
    // Add and update
    
    // Borrow code from the crawler to download the files...
    protected void createCacheFile(String u)
	throws java.net.MalformedURLException
    {
	URL url = new URL(u);
	try {
	    if (preferences.cache.containsKey(u)) {
		// Get the cache file name and tmp file name
		String filename = (new URL((String)preferences.cache.get(u))).getFile();
		String tmpfilename = filename+".tmp";
		File file = new File(filename);
		File tmpfile = new File(tmpfilename);
		
		// Download file (to tmp)
		tmpfile.createNewFile();
		Object content = url.getContent();
		if (content instanceof InputStream) {
		    BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(tmpfile));
		    BufferedInputStream in = new BufferedInputStream((InputStream)content);
		    for (int b = in.read(); b != -1; b = in.read()) {
			out.write(b);
		    }
		    out.flush();
		    out.close();
		
		    // Delete old cache file
		    try {
			if (file.delete()) {
			    // Rename new file
			    try {
				if (tmpfile.renameTo(file)) {
				    System.out.println("Updated cache for "+u);
				} else {
				    System.out.println("Unable to rename temporary cache file to "+filename);
				}
			    } catch (SecurityException ex) {
				System.out.println("Unable to rename temporary cache file to "+filename);
				System.out.println(ex);
			    }
			} else {
			    System.out.println("Unable to delete old cache file "+filename);
			}
		    } catch (SecurityException ex) {
			System.out.println("Unable to delete old cache file "+filename);
			System.out.println(ex);
		    }
		} else {
		    System.out.println("Could not update cache for "+u);
		    System.out.println(" Unknown content type.");
		}

	    } else {

		// Not already in cache - create a new cache file
		// Generate cache filename from URI
		File file = new File(cachedir+convertURLtoFile(url));
		// Download URI to filename
		Object content = url.getContent();
		// Create cache mapping
		if (content instanceof InputStream) {
		    // Copy to local file
		    BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(file));
		    BufferedInputStream in = new BufferedInputStream((InputStream)content);
		    for (int b = in.read(); b != -1; b = in.read()) {
			out.write(b);
		    }
		    out.flush();
		    out.close();
		    // Add to preference file
		    preferences.addToCache(u, "file:"+file.toString());
		    System.out.println("Created cache file for "+u);
		} else {
		    System.out.println("Could not add cache file for "+u);
		    System.out.println(" Unknown content type.");
		}
	    }
	} catch (java.io.IOException ex) {
	    System.out.println("Error occurred while processing -add items:");
	    System.out.println(ex);
	}
    }

    /** 
	Create a filename from the host and path.  Convert / and . to _ 
	except for .'s in the filename.
    */
    protected String convertURLtoFile(URL url) 
    {
	StringBuffer filename = new StringBuffer(url.getHost()+url.getPath());
	boolean replace = false;
	for (int i=filename.length()-1; i>=0; i--) {
	    if (replace && (filename.charAt(i)=='.')) {
		filename.setCharAt(i, '_');
	    } else if (filename.charAt(i) == '/') {
		filename.setCharAt(i, '_');
		replace = true;
	    }
	}
	//System.out.println("DEBUG: "+url+" -> "+filename);
	return filename.toString();
    }

    protected void deleteCacheFile(String u) 
	throws java.net.MalformedURLException
    {
	if (preferences.cache.containsKey(u)) {
	    String filename = (new URL((String)preferences.cache.get(u))).getFile();
	    File file = new File(filename);
	    preferences.removeFromCache(u);
	    if (file.isFile()) {
		try {
		    file.delete();
		    System.out.println("Deleted Cache File for "+u);
		} catch (SecurityException ex) {
		    System.out.println("Can't delete Cache File "+file);
		}
	    } else {
		System.out.println("Not a file: "+file.toString());
	    }
	} else {
	    System.out.println("Not in Cache: "+u);
	}
    }

   
}

