// $Id: LoadIndication.java,v 1.2 2003/03/12 21:31:10 drager Exp $

package org.daml.validator.indications;

import com.hp.hpl.mesa.rdf.jena.model.*;
import java.util.*;

/**
 * This indication is used to notify the user what files were substituted or loaded from the cache.
 */
public class LoadIndication extends Indication {
    
    public static int defaultLevel = Indication.INFORMATION;

    /**
     * Collection that holds the list of loaded files.
     */
    static Collection files = null;
    
    boolean recurseNS;
    boolean recurseImports;
    int maxRecurseNS;
    int maxRecurseImports;

    /**
     * Constructor for a CacheIndication.
     * @param files 	The list of Files that were loaded
     */
    public LoadIndication(Collection files, boolean recurseNS, int maxRecurseNS, boolean recurseImports, int maxRecurseImports)
    {
	super();
	type = "Loaded Files";
	level = defaultLevel;
	this.files = files;
	this.recurseNS = recurseNS;
	this.recurseImports = recurseImports;
	this.maxRecurseNS = maxRecurseNS;
	this.maxRecurseImports = maxRecurseImports;
    }
					
    public String getMessage()
    {
	String m = "";

	m = "The following files were loaded by the Validator to support validation:\n";
	for (Iterator i = files.iterator(); i.hasNext(); ){
	    m += i.next()+"\n";
	}

	m += "\nRecursive loading of imported ontologies is ";
	if (recurseImports) {
	    m += "ON -- maximum depth is ";
	    if (maxRecurseImports < 0)
		m += "infinite";
	    else
		m += maxRecurseImports;
	} else {
	    m += "OFF";
	}

	m += "\nRecursive loading of referenced namespaces is ";
	if (recurseNS) {
	    m += "ON -- maximum depth is ";
	    if (maxRecurseNS < 0) 
		m += "infinite";
	    else
		m += maxRecurseNS;
	} else {
	    m += "OFF";
	}

	return(m);
    }

    public String getLocation()
    {
	return("");
    }

    public String getLocationXML()
	throws RDFException
    {
	return("");
    }
}
