// $Id: UndefinedPropertyIndication.java,v 1.2 2002/08/05 19:03:09 drager Exp $

package org.daml.validator.indications;

import com.hp.hpl.mesa.rdf.jena.model.*;
import java.util.*;

/**
 * This indication is used when there is a resource that is
 * referenced but does not exist.
 */
public class UndefinedPropertyIndication extends Indication {
    
    public static int defaultLevel = Indication.INFORMATION;

    /**
     * The undefined predicate.
     */
    Resource resource;
    
    /**
     * The collection of statements
     */
    Collection statements;
    
    /**
     * Constructor for an UndefinedPropertyIndication.
     * @param resource		The inplicit property
     * @param statements	Collection of statements referencing the property
     */
    public UndefinedPropertyIndication(Resource resource, Collection statements)
    {
	super();
	type = "Implicit Property";
	level = defaultLevel;
	this.resource = resource;
	this.statements = statements;
    }
					
    /**
     * Constructor for an UndefinedPropertyIndication.
     * @param resource		The implicit property
     * @param statement		Statement referencing the property
     */
    public UndefinedPropertyIndication(Resource resource, Statement statement)
    {
	super();
	type = "Implicit Property";
	level = defaultLevel;
	this.resource = resource;
	HashSet h = new HashSet();
	h.add(statement);
	this.statements = h;
    }

					
    public String getMessage()
    {
	String m = "";
	m = "Property " + resource + " is not explicitly defined at the specified namespace URI.";
	return(m);
    }

    public String getLocation()
    {
	return(getLocation(statements));
    }

    public String getLocationXML()
	throws RDFException
    {
	return(getLocationXML(statements));
    }
}
